#ifndef __SOUND_H
#define __SOUND_H
#include "stm32f10x.h"

unsigned int duration_ms;
unsigned int note_freq_hz;

#define whole 1000	//whole note, 1 sec duration
#define half 500	//half note, 0.5 sec duration
#define quarter 250	//quarter note, 0.25 sec duration
#define eighth 125	//eighth note, 0.125 sec duration
#define triplet 167	//quarter note, triplet, 0.167 sec duration
#define sixteenth 63 //sixteenth note

//Sets specific note frequency for DAC timer
void Sound_Play(unsigned int freq, unsigned int ms)
{
	duration_ms = ms;
	note_freq_hz = (freq/2);
}

//Note Functions
void A4flat(unsigned int duration)
{
	Sound_Play(0x4B3f,duration);
}
void A4(unsigned int duration)
{
	Sound_Play(0x4705,duration);
}
void B4flat(unsigned int duration)
{
	Sound_Play(0x4309,duration);
}
void B4(unsigned int duration)
{
	Sound_Play(0x3F46,duration);
}
void C5(unsigned int duration)
{
	Sound_Play(0x3BB9,duration);
}
void C5sharp(unsigned int duration)
{
	Sound_Play(0x385E,duration);
}
void D5(unsigned int duration)
{
	Sound_Play(0x3534,duration);
}
void E5flat(unsigned int duration)
{
	Sound_Play(0x3238,duration);
}
void E5(unsigned int duration)
{
	Sound_Play(0x2F66,duration);
}
void F5(unsigned int duration)
{
	Sound_Play(0x2CBD,duration);
}
void F5sharp(unsigned int duration)
{
	Sound_Play(0x2A3A,duration);
}
void G5(unsigned int duration)
{
	Sound_Play(0x27DC,duration);
}
//==================================
void A3flat(unsigned int duration)
{
	Sound_Play(0x967E,duration);
}
void A3(unsigned int duration)
{
	Sound_Play(0x8E0B,duration);
}
void B3flat(unsigned int duration)
{
	Sound_Play(0x8612,duration);
}
void B3(unsigned int duration)
{
	Sound_Play(0x7E8C,duration);
}
void C4(unsigned int duration)
{
	Sound_Play(0x7771,duration);
}
void C4sharp(unsigned int duration)
{
	Sound_Play(0x70BE,duration);
}
void D4(unsigned int duration)
{
	Sound_Play(0x6A6A,duration);
}
void E4flat(unsigned int duration)
{
	Sound_Play(0x6470,duration);
}
void E4(unsigned int duration)
{
	Sound_Play(0x5ECD,duration);
}
void F4(unsigned int duration)
{
	Sound_Play(0x597B,duration);
}
void F4sharp(unsigned int duration)
{
	Sound_Play(0x5476,duration);
}
void G4(unsigned int duration)
{
	Sound_Play(0x4FB8,duration);
}
//==================================
void A2flat(unsigned int duration)
{
	Sound_Play(0x12CF9,duration);
}
void A2(unsigned int duration)
{
	Sound_Play(0x11C17,duration);
}
void B2flat(unsigned int duration)
{
	Sound_Play(0x10C25,duration);
}
void B2(unsigned int duration)
{
	Sound_Play(0xFD19,duration);
}
void C3(unsigned int duration)
{
	Sound_Play(0xEEE5,duration);
}
void C3sharp(unsigned int duration)
{
	Sound_Play(0xE17C,duration);
}
void D3(unsigned int duration)
{
	Sound_Play(0xD4D4,duration);
}
void E3flat(unsigned int duration)
{
	Sound_Play(0xC8E3,duration);
}
void E3(unsigned int duration)
{
	Sound_Play(0xBD9C,duration);
}
void F3(unsigned int duration)
{
	Sound_Play(0xB2F8,duration);
}
void F3sharp(unsigned int duration)
{
	Sound_Play(0xA8EB,duration);
}
void G3(unsigned int duration)
{
	Sound_Play(0x9F70,duration);
}
void Note_Off()
{
	TIM4->CR1 = 0x10;
}

//Sets frequency to DAC
void Note_Setup()
{
	TIM4->CR1 = 0x10;
	TIM4->ARR = note_freq_hz;
	TIM4->CR1 |=0x1;
	delay_ms(duration_ms);
	TIM4->CR1 = 0x10;
	delay_ms(10);
}

//Super Mario Bros. (1985)
// Overworld / Main Theme Intro
// Modification from original composition
//  by Koji Kondo
void Mario_Main()
{
	E5(quarter);	Note_Setup();		
	E5(quarter);	Note_Setup();	delay_ms(quarter);						
	E5(quarter);	Note_Setup();				
	Note_Off();	delay_ms(quarter);
			
	C5(quarter);	Note_Setup();				
	E5(quarter);	Note_Setup();				
	Note_Off();	delay_ms(quarter);
			
	G5(quarter);	Note_Setup();				
	Note_Off();	delay_ms(half+quarter);
			
	G4(quarter);	Note_Setup();				
	Note_Off();	delay_ms(half+quarter);				
}

//Super Mario Bros. (1985)
// Underworld Theme
// Modification from original composition
//  by Koji Kondo
void Mario_Underground()
{
	
	C4(quarter);	Note_Setup();	
	C5(quarter);	Note_Setup();	
	A3(quarter);	Note_Setup();	
	A4(quarter);	Note_Setup();	
	B3flat(quarter);	Note_Setup();
	B4flat(quarter);	Note_Setup();	
	Note_Off();	delay_ms(whole+half);
	
	
	C4(quarter);	Note_Setup();	
	C5(quarter);	Note_Setup();	
	A3(quarter);	Note_Setup();	
	A4(quarter);	Note_Setup();	
	B3flat(quarter);	Note_Setup();	
	B4flat(quarter);	Note_Setup();	
	Note_Off();	delay_ms(whole+half);


	F3(quarter);	Note_Setup();	
	F4(quarter);	Note_Setup();	
	D3(quarter);	Note_Setup();	
	D4(quarter);	Note_Setup();	
	E3flat(quarter);	Note_Setup();	
	E4flat(quarter);	Note_Setup();	
	Note_Off();	delay_ms(whole+half);

	
	F3(quarter);	Note_Setup();	
	F4(quarter);	Note_Setup();	
	D3(quarter);	Note_Setup();	
	D4(quarter);	Note_Setup();	
	E3flat(quarter);	Note_Setup();	
	E4flat(quarter);	Note_Setup();	
	Note_Off();	delay_ms(whole);

	
	E3flat(triplet);	Note_Setup();	
	D4(triplet);	Note_Setup();	
	C4sharp(triplet);	Note_Setup();	
	C4(half);	Note_Setup();	
	E4flat(half);	Note_Setup();	
	D4(half);	Note_Setup();	
	A3flat(half); 	Note_Setup();	
	G3(half);	Note_Setup();	
	C4sharp(half);	Note_Setup();	


	C4(triplet);	Note_Setup();	
	F4sharp(triplet);	Note_Setup();	
	F4(triplet);	Note_Setup();	
	E4(triplet);	Note_Setup();	
	B4flat(triplet);	Note_Setup();	
	A4(triplet);	Note_Setup();
	
	A4flat(triplet*2);	Note_Setup();	
	E4flat(triplet*2);	Note_Setup();	//flat?
	B3(triplet*2);	Note_Setup();	
	B3flat(triplet*2);	Note_Setup();	//flat?
	A3(triplet*2);	Note_Setup();	
	A3flat(triplet*2);	Note_Setup();	

	Note_Off();	delay_ms(3*whole);
}

//Win Sound Function
void Win_Sound()
{
D4(sixteenth); Note_Setup();
D4(sixteenth); Note_Setup();
D4(eighth); Note_Setup();
C4(sixteenth); Note_Setup();
C4(sixteenth); Note_Setup();
C4(eighth); Note_Setup();
B4flat(sixteenth); Note_Setup();
B4flat(sixteenth); Note_Setup();
B4flat(eighth); Note_Setup();
C4(eighth+half+quarter); Note_Setup();
delay_ms(whole);
}

#endif
